@@ -50,7 +50,9 @@
 #' @param sigma Bandwidth parameter, shorthand for sigma squared. Default: sigma <- ncol(X). Since x variables are standardized, facilitates interprepation of the Gaussian kernel, exp(-dist(X)^2/sigma) a.k.a the similarity score. Of course, if dist between observation i and j is 0, there similarity is 1 since exp(0) = 1. Suppose i and j differ by one standard deviation on each dimension. Then the similarity is exp(-ncol(X)/sigma) = exp(-1) = 0.368.  
 #' @param derivative Logical: Estimate derivatives (as opposed to just coefficients)? Recommended for interpretability.
 #' @param which.derivatives Optional. For which columns of X should marginal effects be estimated ("variables of interest"). If derivative=TRUE and which.derivative=NULL, all will marginal effects estimated (default settings). Example: out = bigKRLS(..., which.derivatives = c(1, 3, 5))
-#' @param vcov.est Logical: Estimate variance covariance matrix? Required to obtain derivatives and standard errors on predictions (default = TRUE).
+#' @param vcov.est Logical: Estimate variance covariance matrix? Required to obtain derivatives and standard errors on predictions. Default is TRUE.
+#' @param Neig Number of eigenvectors and eigenvalues to calculate. The default is to calculate all N and only use those where eigval >= 0.01 max(eigval) (see eigtrunc). See out$eigenvalues and out$lastkeeper. When out$lastkeeper is much smaller than N, set Neig to be approximately out$lastkeeper for similar models, data, etc. to decrease runtime.
+#' @param eigtrunc Eigentruncation, default 0.001. eigtrunc = 0.25 keeps only those eigenvectors/values such that the eigenvalue is at least 25\% of the max. If eigtrunc == 0 (default), all Neig are used to select lambda and to estimate variances.
 #' @param lambda Regularization parameter. Default: estimated based (in part) on the eigenvalues of the kernel via Golden Search with convergence parameter "tolerance." Must be positive, real number. 
 #' @param L Lower bound of Golden Search for lambda. 
 #' @param U Upper bound of Golden Search for lambda.
@@ -74,8 +76,9 @@
 #'# out2 <- bigKRLS(y, X, which.derivatives = 2) 
 #'# if x2 is variable of interest 
 #' @export
-bigKRLS <- function (y = NULL, X = NULL, sigma = NULL, derivative = TRUE, 
-                     which.derivatives = NULL, vcov.est = TRUE, 
+bigKRLS <- function (y = NULL, X = NULL, sigma = NULL, 
+                     derivative = TRUE, which.derivatives = NULL, vcov.est = TRUE,
+                     Neig = NULL, eigtrunc = 0.001,
                      lambda = NULL, L = NULL, U = NULL, tol = NULL,
                      model_subfolder_name = NULL, 
                      overwrite.existing = FALSE, Ncores = NULL, 
@@ -170,10 +173,14 @@ bigKRLS <- function (y = NULL, X = NULL, sigma = NULL, derivative = TRUE,
   # only possible + nontrivial when ncol(X) > 2 
   acf <- acf & p > 2
   
+  Neig <- if(is.numeric(Neig)) min(n, as.integer(Neig)) else n
+  if(!is.numeric(eigtrunc) | eigtrunc < 0 | eigtrunc > 1)
+    stop("eigtrunc must be between 0 (no truncation) and 1 (keep largest only).")
+  
   if(!is.null(which.derivatives)){
-    if(!derivative){
+    
+    if(!derivative)
       stop("which.derivative requires derivative = TRUE\n\nDerivative is a logical indicating whether derivatives should be estimated (as opposed to just coefficients); which.derivatives is a vector indicating which one (with NULL meaning all).")
-    }
     stopifnot(sum(which.derivatives %in% 1:p) == length(which.derivatives))
     if(noisy){
       cat("\nMarginal effects will be calculated for the following x variables:\n")
@@ -181,44 +188,30 @@ bigKRLS <- function (y = NULL, X = NULL, sigma = NULL, derivative = TRUE,
     }
   }
   
-  if (min(X.init.sd) == 0) {
-    stop(paste("The following columns in X are constant and must be removed:",
-               which(X.init.sd == 0)))
-  }
-  
-  if (n != nrow(y)) { stop("nrow(X) not equal to number of elements in y.")}
-  if (colna(y) > 0) { stop("y contains missing data.") }
-  if (colsd(y) == 0) { stop("y is a constant.") }
-  
-  if(!is.null(lambda)){
+  if(min(X.init.sd) == 0) 
+    stop("The following columns in X are constant and must be removed: ", which(X.init.sd == 0))
+  if(n != nrow(y)) 
+    stop("nrow(X) not equal to number of elements in y.")
+  if(colna(y) > 0) 
+    stop("y contains missing data.") 
+  if(colsd(y) == 0) 
+    stop("y is a constant.") 
+  if(!is.null(lambda))
     stopifnot(is.vector(lambda), length(lambda) == 1, is.numeric(lambda), lambda > 0)
-    if(noisy){cat("Using user-inputted value of lambda:", lambda, ".\n")}
-  }
-  
-  if(!is.null(sigma)){
+  if(!is.null(sigma))
     stopifnot(is.vector(sigma), length(sigma) == 1, is.numeric(sigma), sigma > 0)
-  }
-  sigma <- ifelse(is.null(sigma), p, sigma)
+  
+  sigma <- if(is.null(sigma)) p else sigma
   
   if (is.null(tol)) { # tolerance parameter for lambda search
     tol <- n/1000
-    if(noisy){cat("\nUsing default tolerance parameter, n/1000 = ", 
-                  tol, ".\n", sep='')}
   } else {
     stopifnot(is.vector(tol), length(tol) == 1, is.numeric(tol), tol > 0)
-    if(noisy){cat("\nUsing user-inputted tolerance parameter:", tol, ".\n")}
   }
   
-  # removing eigentruncation option for now - re-add soon
-  eigtrunc <- NULL
-  #if (!is.null(eigtrunc) && (!is.numeric(eigtrunc) | eigtrunc > n | eigtrunc < 0)) {
-  #  stop("eigtrunc, if used, must be a number between 0 and N indicating the number of eigenvalues to be used.")
-  #}
-  
   stopifnot(is.logical(derivative), is.logical(vcov.est))
-  if (derivative & !vcov.est) { 
+  if (derivative & !vcov.est) 
     stop("vcov.est is needed to get derivatives (derivative==TRUE requires vcov.est=TRUE).")
-  }
   
   x.is.binary <- apply(X, 2, function(x){length(unique(x))}) == 2 
   if(noisy & sum(x.is.binary) > 0){
@@ -235,26 +228,25 @@ bigKRLS <- function (y = NULL, X = NULL, sigma = NULL, derivative = TRUE,
   y[,1] <- (y[,1] - mean(y[,1]))/sd(y[,1])
   
   # by default uses the same number of cores as X variables or N available - 2, whichever is smaller
-  Ncores <- ifelse(is.null(Ncores), 
-                   min(c(parallel::detectCores() - 2, ncol(X))), Ncores)
-  if(noisy){cat(Ncores, "cores will be used.\n")}
+  Ncores <- if(is.null(Ncores)) min(c(parallel::detectCores() - 2, ncol(X))) else Ncores
+  if(noisy) cat(Ncores, "cores will be used.\n")
   
-  if(noisy){cat("\nStep 1/5: Kernel (started at ", format(Sys.time(), format = "%H:%M:%S"), ").", sep="")}
+  if(noisy) cat("\nStep 1/5: Kernel (started at ", Time(), ").", sep="")
   
-  K <- NULL  # K is the kernel
   K <- bGaussKernel(X, sigma)
+
+  if(noisy) cat("\nStep 2/5: Spectral Decomposition (started at ", Time(), ").", sep="")
   
-  if(noisy){cat("\nStep 2/5: Spectral Decomposition (started at ", 
-                format(Sys.time(), format = "%H:%M:%S"), ").", sep="")}
-  
-  Eigenobject <- bEigen(K, eigtrunc)
+  Eigenobject <- bEigen(K, Neig, eigtrunc)
+
   w[["K.eigenvalues"]] <- Eigenobject$values
+  w[["lastkeeper"]] <- Eigenobject$lastkeeper
   
   if (is.null(lambda)) {
     if(noisy){cat("\nStep 3/5: Golden Search for regularization parameter lambda (started at ", 
-                  format(Sys.time(), format = "%H:%M:%S"), ").", sep="")}
-    lambda <- bLambdaSearch(L = L, U = U, y = y, Eigenobject = Eigenobject, 
-                            eigtrunc = eigtrunc, noisy = noisy)
+                  Time(), ").", sep="")}
+    lambda <- bLambdaSearch(L = L, U = U, y = y, 
+                            Eigenobject = Eigenobject, noisy = noisy)
   }else{
     if(noisy){cat("\nSkipping step 3/5, proceeding with user-inputted lambda.\n")}
   }
@@ -263,9 +255,9 @@ bigKRLS <- function (y = NULL, X = NULL, sigma = NULL, derivative = TRUE,
   if(noisy){cat("Effective Sample Size: ", w[["Neffective"]], '.', sep='')}
   
   if(noisy){cat("\n\nStep 4/5: Calculate coefficients & related estimates (started at ", 
-                format(Sys.time(), format = "%H:%M:%S"), ").", sep="")}
+                Time(), ").", sep="")}
   
-  out <- bSolveForc(y = y, Eigenobject = Eigenobject, lambda = lambda, eigtrunc = eigtrunc)
+  out <- bSolveForc(y = y, Eigenobject = Eigenobject, lambda = lambda)
   
   # bSolveForc obtains the vector of coefficients (weights) 
   # that assign importance to the similarity scores (found in K)
@@ -275,28 +267,17 @@ bigKRLS <- function (y = NULL, X = NULL, sigma = NULL, derivative = TRUE,
   if (vcov.est == TRUE) {
     sigmasq <- bCrossProd(y - yfitted)[]/n
     if(noisy){cat("\nIn standardized units, sigmasq = ", round(sigmasq, 5), ".", sep='')}
-    if (is.null(eigtrunc)) {  # default
-      if(noisy){cat("\nCalculating variance-covariance of the coefficients.")}
-      m <- bMultDiag(Eigenobject$vectors, 
-                     sigmasq * (Eigenobject$values + lambda)^-2)
-      vcovmatc <- bTCrossProd(m, Eigenobject$vectors)
-      
-    }else{
-      
-      lastkeeper = max(which(Eigenobject$values >= eigtrunc * Eigenobject$values[1]))
-      if(noisy){cat("\nCalculating variance-covariance of the coefficients.")}
-      m <- bMultDiag(sub.big.matrix(Eigenobject$vectors, 
-                                    firstCol = 1, 
-                                    lastCol = lastkeeper), 
-                     sigmasq * (Eigenobject$values[1:lastkeeper] + lambda)^-2)
-      vcovmatc <- bTCrossProd(m, sub.big.matrix(Eigenobject$vectors, 
-                                                firstCol = 1, 
-                                                lastCol = lastkeeper))
-    }
+    if(noisy){cat("\nCalculating variance-covariance of the coefficients.")}
+    
+    # subsetting from Neig < N and/or eigtrunc now handled by bEigen()
+    m <- bMultDiag(Eigenobject$vectors, 
+                   sigmasq * (Eigenobject$values + lambda)^-2)
+    vcovmatc <- bTCrossProd(m, Eigenobject$vectors)
+    
     remove(Eigenobject)
     remove(m)
     gc()
-    if(noisy){"\nEstimating variance covariance of the fitted values."}
+    if(noisy) cat("\nEstimating variance covariance of the fitted values.")
     vcovmatyhat <- bCrossProd(K, vcovmatc %*% K)
     if(!is.null(model_subfolder_name) & return.big.squares){
       vcovmatyhat <- (y.init.sd^2) * vcovmatyhat
@@ -311,26 +292,19 @@ bigKRLS <- function (y = NULL, X = NULL, sigma = NULL, derivative = TRUE,
     vcov.est.c <- NULL
     vcov.est.fitted <- NULL
   }
-  
   if (derivative == TRUE) {
     
     if(noisy){cat("\n\nStep 5/5: Estimate marginal effects and their uncertainty (started at ", 
-                  format(Sys.time(), format = "%H:%M:%S"), ").\n\n", sep="")} 
+                  Time(), ").\n\n", sep="")}
+    
+    X_estimate <- if(!is.null(which.derivatives)) deepcopy(X, cols = which.derivatives) else X
     
     if(Ncores == 1){
-      if(is.null(which.derivatives)){
-        deriv_out <- bDerivatives(X, sigma, K, out$coeffs, vcovmatc, X.init.sd)
-      }else{
-        Xsubset <- deepcopy(X, cols = which.derivatives)
-        deriv_out <- bDerivatives(Xsubset, sigma, K, out$coeffs, vcovmatc, X.init.sd)
-      }
+        deriv_out <- bDerivatives(X_estimate, sigma, K, out$coeffs, vcovmatc)
     }else{
-      if(is.null(which.derivatives)){
-        delta <- 1:p
-      }else{
-        delta <- which.derivatives
-      }
       
+      X_index <- if(is.null(which.derivatives)) 1:p else which.derivatives
+
       # each core will need to know how to find the big matrices
       # writing their description to disk will allow each core to do that...
       
@@ -342,7 +316,7 @@ bigKRLS <- function (y = NULL, X = NULL, sigma = NULL, derivative = TRUE,
         clusterEvalQ(cl, suppressPackageStartupMessages(library(bigKRLS)))
       } 
       
-      tmp <- parLapply(cl, delta, function(i, sigma, coefficients, X.init.sd, path){
+      tmp <- parLapply(cl, X_index, function(i, sigma, coefficients, X.init.sd, path){
 
         # each core finds the big matrices like so...
         X <- attach.resource(dget(file.path(path, "X.desc")), path = path)
@@ -355,7 +329,7 @@ bigKRLS <- function (y = NULL, X = NULL, sigma = NULL, derivative = TRUE,
           
         x <- deepcopy(X, cols = i)
         
-        output <- bDerivatives(x, sigma, K, coefficients, V, X.init.sd)
+        output <- bDerivatives(x, sigma, K, coefficients, V)
         # can't return pointers
         list(output[[1]][], output[[2]])
         # output is small but could also use to.big.matrix for reverse direction (should if N * N needed)
@@ -363,9 +337,9 @@ bigKRLS <- function (y = NULL, X = NULL, sigma = NULL, derivative = TRUE,
       stopCluster(cl) 
       remove(cl)
       
-      derivs <- matrix(nrow = n, ncol = length(delta))
+      derivs <- matrix(nrow = n, ncol = ncol(X_estimate))
       varavgderiv <- c()
-      for(i in 1:length(delta)){
+      for(i in 1:ncol(X_estimate)){
         derivs[,i] <- tmp[[i]][[1]]
         varavgderiv[i] <- tmp[[i]][[2]]
       }
@@ -377,7 +351,7 @@ bigKRLS <- function (y = NULL, X = NULL, sigma = NULL, derivative = TRUE,
     
     
     if(noisy){
-      cat('\n\nFinshed (', format(Sys.time(), format = "%H:%M:%S"), ').', sep="") 
+      cat('\n\nFinshed (', Time(), ').', sep="") 
       cat('\n\nPrepping bigKRLS output object...\n')
     }
     
@@ -385,6 +359,11 @@ bigKRLS <- function (y = NULL, X = NULL, sigma = NULL, derivative = TRUE,
     varavgderivmat <- deriv_out$varavgderiv
     remove(deriv_out)
     
+    # Pseudo R2 using only Average Marginal Effects
+    yhat_ame <- (X_estimate[] %*% colMeans(derivmat[]))^2
+
+    w[["R2AME"]] <- cor(y.init[], yhat_ame)^2
+    
     derivmat <- y.init.sd * derivmat
     for(i in 1:ncol(derivmat)){
       derivmat[,i] <- derivmat[,i]/X.init.sd[i]
@@ -448,14 +427,7 @@ bigKRLS <- function (y = NULL, X = NULL, sigma = NULL, derivative = TRUE,
   
   w[["derivative.call"]] <- derivative
   
-  if(derivative){
-    # Pseudo R2 using only Average Marginal Effects
-    if(is.null(which.derivatives)){
-      w[["R2AME"]] <- cor(y.init[], (X %*% matrix(avgderiv, ncol=1))[])^2
-    }else{
-      w[["R2AME"]] <- cor(y.init[], (X[,which.derivatives] %*% matrix(avgderiv, ncol=1))[])^2
-    }
-    
+if(derivative){
     rownames(avgderiv) <- rownames(varavgderivmat) <- ""
     
     w[["avgderivatives"]] <- avgderiv
@@ -668,7 +640,9 @@ summary.bigKRLS <- function (object, degrees = "Neffective", probs = c(0.05, 0.2
   
   stopifnot(degrees %in% c("acf", "Neffective", "N"))
   
-  if(degrees == "Neffective") n <- object$Neffective
+  if(degrees == "Neffective"){
+    n <- object$Neffective
+  }
   if(degrees == "acf"){
     if(is.null(object$Neffective.acf)){
       big.meta <- create.metadata.dir()
@@ -693,6 +667,9 @@ summary.bigKRLS <- function (object, degrees = "Neffective", probs = c(0.05, 0.2
     return(invisible(NULL))
   }
   
+  if(!is.null(object$R2AME))
+    cat("R2AME**:", round(object$R2AME, digits), "\n\n")
+  
   if(!is.null(labs)){
     stopifnot(length(labs) == p)
     colnames(object$X) <- labs
@@ -701,7 +678,6 @@ summary.bigKRLS <- function (object, degrees = "Neffective", probs = c(0.05, 0.2
     colnames(object$X) <- object$xlabs
   }
   
-  cat("R2AME**:", round(object$R2AME, digits), "\n\n")
   if(is.null(object$which.derivatives)){
     object$which.derivatives <- 1:p
   }
@@ -809,7 +785,7 @@ summary.bigKRLS_CV <- function (object, ...)
     print(round(overview, digits = digits), na.print = "")
     
     cat("\n\nSummary of Training Model:\n")
-    z = summary(object$trained, ...)
+    z <- summary(object$trained, ...)
     
     ans <- list(overview = overview,
                 training.ttests = z$ttests, 
@@ -851,7 +827,7 @@ summary.bigKRLS_CV <- function (object, ...)
     
     for(k in 1:object$Kfolds){
       cat("\n\nSummary of Training Model", k , ":\n", sep="")
-      z = summary(object[[paste0("fold_", k)]][["trained"]], ...)
+      z <- summary(object[[paste0("fold_", k)]][["trained"]], ...)
       ans[[paste0("training", k, ".ttests")]] <- z$ttests
       ans[[paste0("training", k, ".percentiles")]] <- z$percentiles
     }
@@ -1184,7 +1160,8 @@ crossvalidate.bigKRLS <- function(y, X, seed, Kfolds = NULL, ptesting = NULL, es
       delta <- if(is.big.matrix(Xtest)) 
         to.big.matrix(matrix(trained$avgderivatives), p = 1, path = big.meta) else
           t(trained$avgderivatives)
-      cv_out[["pseudoR2AME_oos"]] <- cor(tested[["ytest"]][], (Xtest %*% delta)[])^2
+      yhat_ame <- (Xtest %*% delta)[]
+      cv_out[["pseudoR2AME_oos"]] <- cor(tested[["ytest"]][], yhat_ame)^2
       cv_out[["MSE_AME_oos"]] <- mean((ytest - (Xtest %*% delta)[])^2)
       
     }
@@ -1226,7 +1203,7 @@ crossvalidate.bigKRLS <- function(y, X, seed, Kfolds = NULL, ptesting = NULL, es
     warn.big <- FALSE # dummy flag variable: warn re: big.matrix objects?
     
     # K measures of fit for each fold...
-    out[["R2_is"]] <- c() # in sample R2, based on y = kernel(train, ) %*% coefs.hat
+    out[["R2_is"]] <- c() # in sample R2, based on yhat = kernel[train, ] %*% coefs.hat
     out[["R2_oos"]] <- c() # out of sample R2, based on kernel(X[cbind(test, train])
     out[["MSE_is"]] <- c() # in sample mean squared error
     out[["MSE_oos"]] <- c() # out of sample mean squared error
@@ -1282,9 +1259,9 @@ crossvalidate.bigKRLS <- function(y, X, seed, Kfolds = NULL, ptesting = NULL, es
         delta <- if(is.big.matrix(Xtest)) 
           to.big.matrix(matrix(trained$avgderivatives), p = 1, path = big.meta) else
             t(trained$avgderivatives)
-        out[["R2AME_oos"]][k] <- cor(ytest, (Xtest %*% delta)[])^2
-        out[["MSE_AME_oos"]][k] <- cv_out[["tested"]][["MSE_AME"]] <- mean((ytest - (Xtest %*% delta)[])^2)
-        
+        yhat_ame <- (Xtest %*% delta)[]
+        out[["R2AME_oos"]][k] <- cor(ytest, yhat_ame)^2
+        out[["MSE_AME_oos"]][k] <- cv_out[["tested"]][["MSE_AME"]] <- mean((ytest - yhat_ame)^2)
       }
       
       warn.big <- warn.big | ("big.matrix" %in% lapply(trained, class) & is.null("estimates_subfolder")) 
@@ -1309,431 +1286,3 @@ crossvalidate.bigKRLS <- function(y, X, seed, Kfolds = NULL, ptesting = NULL, es
   }
   
 }
-
-####################################
-# functions that support bigKRLS() #
-####################################
-
-bLambdaSearch <- function (L = NULL, U = NULL, y = NULL, Eigenobject = NULL, tol = NULL, 
-                           noisy = FALSE, eigtrunc = NULL){
-  
-  if(sum(is.na(Eigenobject$values)) > 0) stop("Missing eigenvalues prevent bigKRLS from obtaining the regularization parameter lambda.\n\tCheck for repeated observations (or other perfect linear combinations in X).")
-  n <- nrow(y)
-  if (is.null(tol)) {
-    tol <- 10^-3 * n # tolerance parameter
-  } else {
-    stopifnot(is.vector(tol), length(tol) == 1, is.numeric(tol), tol > 0)
-  }
-  if (is.null(U)) {
-    U <- n
-    while (sum(Eigenobject$values/(Eigenobject$values + U)) < 1) {
-      U <- U - 1
-    }
-  } else {
-    stopifnot(is.vector(U), length(U) == 1, is.numeric(U), U > 0)
-  }
-  if (is.null(L)) {
-    
-    q <- which.min(abs((Eigenobject$values - max(Eigenobject$values)/1000)))
-    L = .Machine$double.eps # smallest double such that 1 + x != 1. Normally 2.220446e-16.
-    
-    while (sum(Eigenobject$values/(Eigenobject$values + L)) > q) {
-      L <- L + 0.05 
-    } 
-  } else {
-    stopifnot(is.vector(L), length(L) == 1, is.numeric(L), L >= 0)
-  }
-  X1 <- L + (0.381966) * (U - L) 
-  X2 <- U - (0.381966) * (U - L)
-  
-  # bLooLoss is big Leave One Out Error Loss
-  
-  if(noisy) cat("\n\nGetting S1.")
-  S1 <- bLooLoss(lambda = X1, y = y, Eigenobject = Eigenobject, 
-                 eigtrunc = eigtrunc)
-  if(noisy){cat("\nGetting S2.")}
-  S2 <- bLooLoss(lambda = X2, y = y, Eigenobject = Eigenobject, 
-                 eigtrunc = eigtrunc)
-  f3 <- function(x){format(round(x, digits=3), nsmall=3)}
-  if (noisy) {
-    cat("\n\nL: ", f3(L), 
-        " X1: ", f3(X1), " X2: ", f3(X2), 
-        " U: ", f3(U), " S1: ", f3(S1), " S2: ", f3(S2), 
-        "\n(", format(Sys.time(), format = "%H:%M:%S"),  
-        ").", sep = "")
-  }
-  while (abs(S1 - S2) > tol) {
-    if (S1 < S2) {
-      U <- X2
-      X2 <- X1
-      X1 <- L + (0.381966) * (U - L)
-      S2 <- S1
-      S1 <- bLooLoss(lambda = X1, y = y, Eigenobject = Eigenobject, 
-                     eigtrunc = eigtrunc)
-    }
-    else {
-      L <- X1
-      X1 <- X2
-      X2 <- U - (0.381966) * (U - L)
-      S1 <- S2
-      S2 <- bLooLoss(lambda = X2, y = y, Eigenobject = Eigenobject, 
-                     eigtrunc = eigtrunc)
-    }
-    if (noisy) {
-      cat("\nL: ", f3(L), 
-          " X1: ", f3(X1), " X2: ", f3(X2), 
-          " U: ", f3(U), " S1: ", f3(S1), " S2: ", f3(S2), 
-          "\n(", format(Sys.time(), format = "%H:%M:%S"),  
-          ").", sep = "")    }
-  }
-  out <- ifelse(S1 < S2, X1, X2)
-  
-  if (noisy) {cat("\n\nlambda = ", round(out, 5), ".\n", sep='')}
-  
-  return(invisible(out))
-}
-
-bSolveForc <- function (y = NULL, Eigenobject = NULL, lambda = NULL, eigtrunc=NULL) {
-  
-  out <- BigSolveForc(Eigenobject$vectors@address, Eigenobject$values, y[], lambda)
-  return(list(Le = out[[1]], coeffs = out[[2]]))
-}
-
-bLooLoss <- function (y = NULL, Eigenobject = NULL, lambda = NULL, eigtrunc = NULL) 
-{
-  return(bSolveForc(y = y, Eigenobject = Eigenobject, lambda = lambda, 
-                    eigtrunc = eigtrunc)$Le)
-} # not sure that there's any point to this function
-# could just make "bLooLoss" mode a parameter of bSolveForc
-
-
-
-#######################################
-# Rcpp and bigmemory Helper Functions #
-#######################################
-
-noise <- function() sample(as.numeric(gsub("\\.", "", as.character(as.numeric(format(Sys.time(), "%OS"))))), 1)
-
-create.metadata.dir <- function(){
-  
-  tmp <- tempdir()
-  Nextant <- length(dir(path = tmp, pattern = "filebacks"))
-  big.meta <- file.path(tmp, paste0("filebacks", Nextant + 1, "_", noise()))   
-  dir.create(big.meta)
-  return(big.meta)
-  
-}
-
-
-to.big.matrix <- function(object, p = NULL, deepcopy = FALSE, name = NULL, path = NULL){
-  
-# returns file-backed big.matrix 
-# (takes `object` as matrix, matrix-like object, or big.matrix)
-# ensures vectors are column matrices
-# coerces integers to numeric so that they are properly cast as doubles on Rcpp side
-# name is optional. 
-# name = "X" stores info on the bigmatrix being created as "X.desc".
-# name = NULL leads to "tmp.desc" (or tmp1, tmp2, etc. as necessary)
-#   
-# optionally returns deep copy  
-  
-  # p <- if(is.null(p) && is.null(ncol(object))) 1 else ncol(object)
-  
-  if(is.null(p)){
-    p <- ifelse(is.null(ncol(object)), 1, ncol(object))
-  } 
-  
-  if(is.null(path)){
-    path <- create.metadata.dir()
-  }
-  
-  if(is.null(name)) {
-    #name <- paste0("bigmatrix", length(dir(path = path, pattern = "desc")) + 1, "_", atomic())
-    name <- basename(tempfile(tmpdir = path))
-  }
-
-  if(!is.big.matrix(object)){
-    object <- as.big.matrix(matrix(as.numeric(object), ncol = p), 
-                                   backingfile = name, 
-                                   backingpath = path,
-                                   descriptorfile = paste0(name, ".desc"))
-  }else{
-   
-    # use filebacked function instead ?
-    dput(describe(object), file.path(path, paste0(name, ".desc")))
-     
-  }
-  if(deepcopy) return(deepcopy(object)) else return(object)
-
-}
-
-
-bMultDiag <- function (X, v, check_platform = FALSE) {
-  
-  if(check_platform) check_platform()
-  # multdiag.cpp
-  out <- big.matrix(nrow = nrow(X),
-                    ncol = ncol(X),
-                    init = 0,
-                    type = 'double')
-  
-  BigMultDiag(X@address, v, out@address)
-  
-  return(out)
-}
-
-bEigen <- function(X, eigtrunc, check_platform = FALSE){
-  
-  if(check_platform) check_platform()
-  # eigen.cpp
-  vals <- big.matrix(nrow = 1,
-                     ncol = ncol(X),
-                     init = 0,
-                     type = 'double')
-  vecs <- big.matrix(nrow = nrow(X),
-                     ncol = ncol(X),
-                     init = 0,
-                     type = 'double')
-  if(is.null(eigtrunc)){
-    eigtrunc <- ncol(X)
-  }
-  
-  BigEigen(X@address, eigtrunc, vals@address, vecs@address)
-  return(list('values' = vals[,], 'vectors' = vecs*-1))
-}
-
-bGaussKernel <- function(X, sigma, check_platform = FALSE){
-  
-  if(check_platform) check_platform()
-  # gauss_kernel.cpp
-  out <- big.matrix(nrow=nrow(X), ncol=nrow(X), init=0)
-  
-  BigGaussKernel(X@address, out@address, sigma)
-  return(out)
-}
-
-bNeffective <- function(X, check_platform = FALSE){
-  
-  if(check_platform) check_platform()
-  # Neffective.cpp
-  return(BigNeffective(X@address))
-}
-
-bTempKernel <- function(X_new, X_old, sigma, check_platform = FALSE){
-  
-  if(check_platform) check_platform()
-  # temp_kernel.cpp
-  out <- big.matrix(nrow=nrow(X_new), ncol=nrow(X_old), init=0)
-  
-  BigTempKernel(X_new@address, X_old@address, out@address, sigma)
-  return(out)
-}
-
-bCrossProd <- function(X, Y=NULL, check_platform = FALSE){
-  
-  if(check_platform) check_platform()
-  if(is.null(Y)){
-    Y <- deepcopy(X)
-  }
-  out <- big.matrix(nrow = ncol(X),
-                    ncol = ncol(Y),
-                    init = 0,
-                    type = 'double')
-  BigCrossProd(X@address, Y@address, out@address)
-  return(out)
-}
-
-bTCrossProd <- function(X, Y = NULL, check_platform = FALSE){
-  
-  if(check_platform) check_platform()
-  if(is.null(Y)){
-    Y <- deepcopy(X)
-  }
-  out <- big.matrix(nrow = nrow(X),
-                    ncol = nrow(Y),
-                    init = 0,
-                    type = 'double')
-  BigTCrossProd(X@address, Y@address, out@address)
-  return(out)
-}
-
-bDerivatives <- function(X, sigma, K, coeffs, vcovmatc, X.sd, check_platform = FALSE){
-  
-  if(check_platform) check_platform()
-  derivatives <- big.matrix(nrow=nrow(X), ncol=ncol(X), init=-1)
-  varavgderiv <- big.matrix(nrow=1, ncol=ncol(X), init=-1)
-  out <- BigDerivMat(X@address, K@address, vcovmatc@address, 
-                     derivatives@address, varavgderiv@address,
-                     X.sd, coeffs, sigma)
-  
-  return(list('derivatives'= derivatives, 'varavgderiv' = varavgderiv[]))
-}
-
-make_path <- function(object, model_subfolder_name, overwrite.existing){
-  
-  # thanks to Peter Foley for helpful suggestions re: file and folder management!
-  # see pulls 11-13 starting with https://github.com/rdrr1990/bigKRLS/pull/11
-  
-  if(!overwrite.existing && dir.exists(model_subfolder_name)){
-    i <- 1
-    tmp.name <- paste(model_subfolder_name, i, sep="")
-    while(tmp.name %in% dir()){
-      tmp.name <- paste(model_subfolder_name, i, sep="")
-      i <- i + 1
-    }
-    if(model_subfolder_name %in% dir()){
-      warning(cat("A subfolder named", model_subfolder_name, "exists in your current working directory. Your output will be saved to", tmp.name, "instead. To turn off this safeguard, set save.bigKRLS(..., overwrite.existing = TRUE) next time.\n\n"))
-    }
-    model_subfolder_name <- tmp.name
-  }
-  
-  dir.create(model_subfolder_name, recursive = TRUE, showWarnings = FALSE)
-  if(dir.exists(model_subfolder_name)) 
-    cat("Saving model estimates to:\n\n", model_subfolder_name, "\n\n") else 
-      stop("Unable to create directory.")
-  object[["path"]] <- normalizePath(model_subfolder_name)
-  object[["model_subfolder_name"]] <- model_subfolder_name
-  return(object)
-  
-}
-
-bSave <- function(object, noisy){
-  
-  is.big.mat <- unlist(lapply(object, is.big.matrix))
-  
-  for(i in which(is.big.mat)){
-    output_path <- file.path(object[["model_subfolder_name"]], paste0(names(object)[i], ".txt"))
-    if(noisy) cat("\twriting", output_path, "...\n")
-    write.big.matrix(x = object[[i]], # col.names = !is.null(colnames(object[[i]])),
-                     filename = output_path)
-  }
-  
-  Nbm <- sum(is.big.mat)
-  if(noisy) cat("\n", Nbm, " matrices saved as big matrices", 
-                ifelse(Nbm == 0, " (base R save() may be used safely in this case too).\n",
-                       ", use load.bigKRLS() on the entire directory to reconstruct the outputted object in R.\n"), sep="")
-  if(Nbm > 0){
-    bigKRLS_out <- object[-which(is.big.mat)]
-    class(bigKRLS_out) <- class(object) 
-  }else{
-    bigKRLS_out <- object
-  }
-  remove(object)
-  stopifnot(sum(unlist(lapply(bigKRLS_out, is.big.matrix))) == 0)
-  save(bigKRLS_out, 
-       file = file.path(bigKRLS_out[["model_subfolder_name"]], "estimates.RData"))
-  if(noisy) cat("Smaller, base R elements of the outputted object saved:", 
-                file.path(bigKRLS_out[["model_subfolder_name"]], "estimates.RData"), "\n")
-  
-}
-
-bLoad <- function(object, path, noisy){
-  
-  options(bigmemory.allow.dimnames=TRUE)
-  
-  if(class(object) == "bigKRLS"){
-    matrices <- c("K", "X", "derivatives", "vcov.est.c", "vcov.est.fitted")
-  }else{
-    if(class(object) == "bigKRLS_predicted"){
-      matrices <- c("predicted", "se.pred", "vcov.est.pred", "newdata", "newdataK", "ytest")
-    }else{
-      stop("bLoad may only be used on bigKRLS objects or bigKRLS_predicted objects.")
-    }
-  }
-  
-  `%out%` <- function(x, table) match(x, table, nomatch = 0L) == 0L
-  xlabs <- object$xlabs
-  which.derivatives <- object$which.derivatives
-  
-  for(i in 1:length(matrices)){
-    
-    filename <- paste0(matrices[i], ".txt")
-    
-    if(filename %out% dir(path = path) & matrices[i] %out% names(object)){
-      
-      if(noisy) cat("NOTE:", matrices[i],  
-                    "not found in .RData or in big matrix file,", 
-                    filename,".\n\n")
-    }else{
-      if(filename %in% dir(path = path)){
-        if(noisy) cat("\tReading from", filename, "\n")
-        object[[matrices[i]]] <- read.big.matrix(file.path(path, filename), 
-                                                 type = "double")
-        
-        if(ncol(object[[matrices[i]]]) == length(xlabs)){
-          colnames(object[[matrices[i]]]) <- xlabs
-        }else{
-          if(!is.null(which.derivatives)){
-            if(ncol(object[[matrices[i]]]) == length(xlabs[which.derivatives])){
-              colnames(object[[matrices[i]]]) <- xlabs[which.derivatives]
-            }
-          }
-        }
-        stopifnot(is.big.matrix(object[[matrices[i]]]))
-      }
-      
-    }
-  }
-  
-  return(object)
-}
-
-# check_data() performs all the checks that bigKRLS() performs...
-# it is intended for K folds crossvalidation. 
-# categorical variables can be fussy when randomly partitioning...
-
-check_data <- function (y = NULL, X = NULL, sigma = NULL, 
-                        derivative = TRUE, which.derivatives = NULL,
-                        vcov.est = TRUE, lambda = NULL, L = NULL, U = NULL, 
-                        tol = NULL, model_subfolder_name = NULL, 
-                        overwrite.existing = FALSE, Ncores = NULL, 
-                        acf = FALSE, noisy = NULL, instructions = TRUE)
-{
-  
-  # suppressing warnings from bigmatrix
-  oldw <- getOption("warn")
-  options(warn = -1)
-  #  options(bigmemory.allow.dimnames=TRUE)
-  
-  stopifnot(is.matrix(X) | is.big.matrix(X))
-  big.meta <- create.metadata.dir()
-  
-  X <- to.big.matrix(X, path = big.meta)
-  X.init.sd <- colsd(X)
-  y <- to.big.matrix(y, p = 1, path = big.meta)
-  
-  miss.ind <- colna(X)
-  if (sum(miss.ind) > 0) { 
-    stop(paste("the following columns in X contain missing data, which must be removed:", 
-               paste((1:length(miss.ind))[miss.ind > 0], collapse = ', '), collapse=''))
-  }
-  n <- nrow(X)
-  p <- ncol(X)
-  
-  if (min(X.init.sd) == 0) {
-    stop(paste("The following columns in X are constant and must be removed:",
-               which(X.init.sd == 0)))
-  }
-  
-  if (n != nrow(y)) { stop("nrow(X) not equal to number of elements in y.")}
-  if (colna(y) > 0) { stop("y contains missing data.") }
-  if (colsd(y) == 0) { stop("y is a constant.") }
-  
-  unlink(big.meta, recursive = TRUE)
-  
-}
-
-bDiag <- function(A){
-  # returns diagonal of big.matrix as column vector (base R matrix)  
-  
-  d <- matrix(nrow = nrow(A), ncol = 1)
-  for(i in 1:nrow(A)){
-    d[i] <- deepcopy(A, cols = i, rows = i)[]
-  }
-  
-  return(d) # make to.big.matrix(d)
-} 
-
-submatrix <- function(X, rows){
-  if(is.big.matrix(X)) deepcopy(X, rows = rows) else X[rows, ]
-} # add cols, to.big.matrix... check on behavior of deep copy with filebacked
