@@ -12,10 +12,7 @@ using namespace arma;
 
 template <typename T>
 void xBigCrossProd(const Mat<T>& A, const Mat<T>& B, Mat<T> out) {
-
-  Mat<T> transposed = trans(A);
-  out = transposed * B;
-
+  out = trans(A) * B;
 }
 
 // [[Rcpp::export]]
@@ -30,15 +27,30 @@ void BigCrossProd(SEXP pA, SEXP pB, SEXP pOut) {
     arma::Mat<double>((double *)xpB->matrix(), xpB->nrow(), xpB->ncol(), false),
     arma::Mat<double>((double *)xpOut->matrix(), xpOut->nrow(), xpOut->ncol(), false)
   );
-
 }
 
+template <typename T>
+void xBigXtX(const Mat<T>& A, Mat<T> out) {
+  out = trans(A) * A;
+}
+
+// [[Rcpp::export]]
+void BigXtX(SEXP pA, SEXP pOut) {
+  
+  XPtr<BigMatrix> xpA(pA);
+  XPtr<BigMatrix> xpOut(pOut);
+  
+  xBigXtX(
+    arma::Mat<double>((double *)xpA->matrix(), xpA->nrow(), xpA->ncol(), false),
+    arma::Mat<double>((double *)xpOut->matrix(), xpOut->nrow(), xpOut->ncol(), false)
+  );
+  
+}
+
+
 template <typename T>
 void xBigTCrossProd(const Mat<T>& A, const Mat<T>& B, Mat<T> out) {
-
-  Mat<T> transposed = trans(B);
-  out = A * transposed;
-
+  out = A * trans(B);
 }
 
 // [[Rcpp::export]]
@@ -54,3 +66,23 @@ void BigTCrossProd(SEXP pA, SEXP pB, SEXP pOut) {
     arma::Mat<double>((double *)xpOut->matrix(), xpOut->nrow(), xpOut->ncol(), false)
   );
 }
+
+template <typename T>
+void xBigXXt(const Mat<T>& A, Mat<T> out) {
+  out = A * trans(A);
+}
+
+// [[Rcpp::export]]
+void BigXXt(SEXP pA, SEXP pOut) {
+  
+  XPtr<SharedMemoryBigMatrix> xpA(pA);
+  XPtr<SharedMemoryBigMatrix> xpOut(pOut);
+  
+  xBigXXt(
+    arma::Mat<double>((double *)xpA->matrix(), xpA->nrow(), xpA->ncol(), false),
+    arma::Mat<double>((double *)xpOut->matrix(), xpOut->nrow(), xpOut->ncol(), false)
+  );
+}
+
+
+
